#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

void scandirectory(const char *prefix, const char *dirname, dev_t dev)
{
	struct dirent **namelist;
	int n;

	if (!prefix)
		prefix = dirname;

	if (chdir(dirname))
		return;

	n = scandir(".", &namelist, 0, alphasort);
	if (n < 0)
		perror("scandir");
	else {
		while (n--) {
			char *name = namelist[n]->d_name;
			struct stat buf;

			lstat(name, &buf);
			if (strcmp(name, ".") && strcmp(name, "..")) {
				if (S_ISDIR(buf.st_mode)) {
					char *newprefix =
						malloc(strlen(prefix) + strlen(name) + 2);
					if (!newprefix)
						exit(1);
					strcpy(newprefix, prefix);
					strcat(newprefix, "/");
					strcat(newprefix, name);
					scandirectory(newprefix, name, dev);
					free(newprefix);
				} else if (S_ISBLK(buf.st_mode) && (buf.st_rdev == dev)) {
					printf("%s/%s", prefix, name);
					exit(0);
				}
			}
			free(namelist[n]);
		}
		free(namelist);
	}

	chdir("..");
}

int main(int argc, char **argv)
{
	struct stat buf;

	if (argc != 2)
		return 1;

	if (stat(argv[1], &buf)) {
		perror("getdev");
		exit(1);
	}
	scandirectory(NULL, "/dev", buf.st_dev);

	fprintf(stderr, "%s: no device (%lld)\n", argv[1], buf.st_dev);
	return 0;
}
